package com.tzld.piaoquan.wecom.job;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.tzld.piaoquan.wecom.common.constant.TimeConstant;
import com.tzld.piaoquan.wecom.common.enums.MessageAttachmentTypeEnum;
import com.tzld.piaoquan.wecom.component.HttpPoolClient;
import com.tzld.piaoquan.wecom.dao.mapper.*;
import com.tzld.piaoquan.wecom.model.bo.ExternalUser;
import com.tzld.piaoquan.wecom.model.bo.MiniprogramRecord;
import com.tzld.piaoquan.wecom.model.bo.SendDetail;
import com.tzld.piaoquan.wecom.model.bo.XxlJobParam;
import com.tzld.piaoquan.wecom.model.po.*;
import com.tzld.piaoquan.wecom.service.AccessTokenService;
import com.tzld.piaoquan.wecom.service.HistoryMessageService;
import com.tzld.piaoquan.wecom.service.MessageAttachmentService;
import com.tzld.piaoquan.wecom.service.UserService;
import com.tzld.piaoquan.wecom.utils.DateUtil;
import com.tzld.piaoquan.wecom.utils.LarkRobotUtil;
import com.tzld.piaoquan.wecom.utils.MessageUtil;
import com.tzld.piaoquan.wecom.utils.page.Page;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

import static com.tzld.piaoquan.wecom.common.constant.MessageConstant.MAX_VIDEO_NUM;
import static com.tzld.piaoquan.wecom.common.constant.WeComConstant.*;
import static com.tzld.piaoquan.wecom.common.enums.SourceEnum.HISTORICAL_TOP;
import static com.tzld.piaoquan.wecom.common.enums.SourceEnum.MANUAL;

@Slf4j
@Component
public class WeComHistoryDataJob {

    @Autowired
    private HttpPoolClient httpPoolClient;

    @Autowired
    private AccessTokenService accessTokenService;

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private HistoryMessageMapper historyMessageMapper;

    @Autowired
    private HistoryMessageService historyMessageService;

    @Autowired
    private MessageAttachmentService messageAttachmentService;

    @Autowired
    private StaffMapper staffMapper;

    @Autowired
    private UserService userService;

    @Autowired
    private CorpMapper corpMapper;

    @Autowired
    private MessageAttachmentMapper messageAttachmentMapper;


    @XxlJob("saveHistoryMessageJob")
    public ReturnT<String> selectHistoryMessageByDay(String param) {
        XxlJobParam xxlJobParam = new XxlJobParam();
        if (StringUtils.isNotEmpty(param)) {
            xxlJobParam = JSONObject.parseObject(param, XxlJobParam.class);
        }
        if (xxlJobParam.getStartTime() == null) {
            HistoryMessageExample example = new HistoryMessageExample();
            example.setOrderByClause("create_time desc");
            example.setPage(new Page<>(1, 1));
            List<HistoryMessage> historyMessages = historyMessageMapper.selectByExample(example);
            if (CollectionUtils.isEmpty(historyMessages)) {
                xxlJobParam.setStartTime(1721404800L);
            } else {
                xxlJobParam.setStartTime(historyMessages.get(0).getCreateTime().getTime() / 1000);
            }

        }
        if (xxlJobParam.getEndTime() == null) {
            xxlJobParam.setEndTime(System.currentTimeMillis() / 1000);
        }
        Long startTime = xxlJobParam.getStartTime();
        Long endTime = xxlJobParam.getEndTime();
        CorpExample example = new CorpExample();
        if (xxlJobParam.getCorpId() != null) {
            example.createCriteria().andIdEqualTo(xxlJobParam.getCorpId());
        }
        List<Corp> corps = corpMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(corps)) {
            return ReturnT.SUCCESS;
        }
        for (Corp corp : corps) {
            for (; startTime < endTime; startTime += TimeConstant.DAY) {
                selectGroupMsgList(startTime, Math.min(startTime + TimeConstant.DAY, endTime), corp.getId(), xxlJobParam.getStaffId());
            }
        }
        for (Corp corp : corps) {
            selectAlertHistoryMessage(xxlJobParam.getStartTime(), xxlJobParam.getEndTime(), corp.getId(), corp.getName());
        }
        return ReturnT.SUCCESS;
    }

    private void selectAlertHistoryMessage(Long startTime, Long endTime, Long corpId, String corpName) {
        StaffExample staffExample = new StaffExample();
        List<Staff> staffList = staffMapper.selectByExample(staffExample);
        ArrayList<Integer> statusList = Lists.newArrayList(0, 1, 2, 3);
        List<SendDetail> sendDetailList = new ArrayList<>();
        for (Staff staff : staffList) {
            SendDetail sendDetail = new SendDetail();
            sendDetail.setRemark(staff.getRemark());
            List<Long> countList = new ArrayList<>();
            for (Integer status : statusList) {
                HistoryMessageExample example = new HistoryMessageExample();
                example.createCriteria().andSendTimeBetween(new Date(startTime * 1000), new Date(endTime * 1000))
                        .andStaffIdEqualTo(staff.getId())
                        .andStatusEqualTo(status)
                        .andIsDeleteEqualTo(0)
                        .andCorpIdEqualTo(corpId);
                long l = historyMessageMapper.countByExample(example);
                countList.add(l / MAX_VIDEO_NUM);
            }
            sendDetail.setSendCountList(countList);
            sendDetailList.add(sendDetail);
        }
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append(corpName).append("\n");
        for (SendDetail sendDetail : sendDetailList) {
            List<Long> sendCountList = sendDetail.getSendCountList();
            if (CollectionUtils.isEmpty(sendCountList)) {
                continue;
            }
            stringBuilder.append(sendDetail.getRemark());
            long count = sendCountList.get(0) + sendCountList.get(1) + sendCountList.get(2) + sendCountList.get(3);
            stringBuilder.append("总发送数量：").append(count).append("   ");
            stringBuilder.append("未发送数量：").append(sendCountList.get(0)).append("   ");
            stringBuilder.append("已发送数量：").append(sendCountList.get(1)).append("   ");
            stringBuilder.append("不是好友发送失败数量：").append(sendCountList.get(2)).append("   ");
            stringBuilder.append("已经收到其他群发消息失败发送数量：").append(sendCountList.get(3)).append("   ");
            stringBuilder.append("\n");
        }
        LarkRobotUtil.sendMessage(stringBuilder.toString());
    }

    private void selectGroupMsgList(Long startTime, Long endTime, Long corpId, Long staffId) {
        delHistoryMessageList(startTime, endTime, corpId, staffId);
        try {
            String cursor = "";
            do {
                String res = getGroupMsgList(startTime, endTime, cursor, corpId);
                JSONObject jsonObject = JSONObject.parseObject(res);
                Integer errCode = jsonObject.getInteger("errcode");
                if (errCode != 0) {
                    LarkRobotUtil.sendMessage("selectGroupMsgList error startTime = " + startTime + "endTime = " + endTime);
                    log.error("selectGroupMsgList error startTime={}, endTime={}", startTime, endTime);
                    return;
                }
                JSONArray groupMsgList = jsonObject.getJSONArray("group_msg_list");
                if (CollectionUtils.isEmpty(groupMsgList)) {
                    continue;
                }
                for (int i = 0; i < groupMsgList.size(); i++) {
                    JSONObject groupMsg = groupMsgList.getJSONObject(i);
                    JSONArray attachments = groupMsg.getJSONArray("attachments");
                    String msgId = groupMsg.getString("msgid");
                    Long createTime = null;
                    if (groupMsg.getLong("create_time") != null) {
                        createTime = groupMsg.getLong("create_time") * 1000;
                    }
                    if (CollectionUtils.isEmpty(attachments)) {
                        continue;
                    }
                    List<MiniprogramRecord> miniprogramRecordList = new ArrayList<>();
                    List<MessageAttachment> messageAttachmentList = new ArrayList<>();
                    for (int j = 0; j < attachments.size(); j++) {
                        JSONObject miniprogram = attachments.getJSONObject(j).getJSONObject("miniprogram");
                        if (miniprogram == null) {
                            continue;
                        }
                        MiniprogramRecord miniprogramRecord = new MiniprogramRecord();
                        MessageAttachment messageAttachment = new MessageAttachment();
                        String title = miniprogram.getString("title");
                        String appid = miniprogram.getString("appid");
                        String page = miniprogram.getString("page");
                        Long videoId = MessageUtil.getVideoId(page);

                        miniprogramRecord.setVideoId(videoId);
                        miniprogramRecord.setAttachmentIdx(j + 1);
                        miniprogramRecordList.add(miniprogramRecord);

                        messageAttachment.setAppid(appid);
                        messageAttachment.setPage(page);
                        messageAttachment.setTitle(title);
                        messageAttachment.setMiniprogramVideoId(videoId);
                        messageAttachment.setType(MessageAttachmentTypeEnum.MINI_PROGRAM.getType());
                        messageAttachmentList.add(messageAttachment);
                    }
                    List<String> carrierIdList = selectGroupMsgTask(msgId, corpId);
                    if (CollectionUtils.isEmpty(carrierIdList)) {
                        continue;
                    }
                    for (String carrierId : carrierIdList) {
                        StaffExample example = new StaffExample();
                        example.createCriteria().andCarrierIdEqualTo(carrierId);
                        List<Staff> staffList = staffMapper.selectByExample(example);
                        if (CollectionUtils.isEmpty(staffList)) {
                            LarkRobotUtil.sendMessage("企微推送报警：carrierId不存在请检查 " + carrierId);
                            continue;
                        }
                        //指定id以后  过滤其他id
                        if (staffId != null) {
                            if (!Objects.equals(staffList.get(0).getId(), staffId)) {
                                continue;
                            }
                        }
                        List<ExternalUser> externalUsers = selectGroupMsgSendResult(msgId, carrierId, corpId);
                        if (CollectionUtils.isEmpty(externalUsers)) {
                            continue;
                        }
                        for (MessageAttachment messageAttachment : messageAttachmentList) {
                            messageAttachment.setStaffId(staffList.get(0).getId());
                        }
                        messageAttachmentService.addMiniProgram(messageAttachmentList, createTime);
                        insertHistoryMessageList(staffList.get(0), externalUsers, miniprogramRecordList, createTime, corpId, startTime, endTime);
                    }
                }
                String nextCursor = jsonObject.getString("next_cursor");
                if (cursor.equals(nextCursor)) {
                    break;
                }
                cursor = nextCursor;
            } while (StringUtils.isNotEmpty(cursor));
        } catch (IOException e) {
            LarkRobotUtil.sendMessage("selectGroupMsgList error" + e);
            log.error("selectGroupMsgList error", e);
        }
    }

    private List<ExternalUser> selectGroupMsgSendResult(String msgId, String carrierId, Long corpId) throws IOException {
        List<ExternalUser> resList = new ArrayList<>();
        String cursor = "";
        do {
            String res = getGroupMsgSendResult(msgId, carrierId, cursor, corpId);
            JSONObject jsonObject = JSONObject.parseObject(res);
            Integer errCode = jsonObject.getInteger("errcode");
            if (errCode != 0) {
                String errmsg = jsonObject.getString("errmsg");
                log.error("selectGroupMsgSendResult error msgId={} carrierId={} errCode={} errmsg={}", msgId, carrierId, errCode, errmsg);
                return resList;
            }
            JSONArray sendList = jsonObject.getJSONArray("send_list");
            for (int i = 0; i < sendList.size(); i++) {
                JSONObject send = sendList.getJSONObject(i);
                ExternalUser externalUser = new ExternalUser();
                externalUser.setExternalUserId(send.getString("external_userid"));
                externalUser.setStatus(send.getInteger("status"));
                if (send.getLong("send_time") != null) {
                    externalUser.setSendTime(send.getLong("send_time") * 1000);
                }
                resList.add(externalUser);
            }
            String nextCursor = jsonObject.getString("next_cursor");
            if (cursor.equals(nextCursor)) {
                break;
            }
            cursor = nextCursor;
        } while (StringUtils.isNotEmpty(cursor));
        return resList;
    }

    private String getGroupMsgSendResult(String msgId, String carrierId, String cursor, Long corpId) throws IOException {
        String accessToken = accessTokenService.getWeComAccessToken(corpId);
        String url = POST_WE_COM_GROUP_MSG_SEND_RESULT
                + "?access_token=" + accessToken;
        JSONObject param = new JSONObject();
        param.put("msgid", msgId);
        param.put("userid", carrierId);
        param.put("limit", 1000);
        if (StringUtils.isNotEmpty(cursor)) {
            param.put("cursor", cursor);
        }
        return httpPoolClient.post(url, param.toJSONString());
    }

    private List<String> selectGroupMsgTask(String msgId, Long corpId) throws IOException {
        List<String> resList = new ArrayList<>();
        String cursor = "";
        do {
            String res = getGroupMsgTask(msgId, cursor, corpId);
            JSONObject jsonObject = JSONObject.parseObject(res);
            Integer errCode = jsonObject.getInteger("errcode");
            if (errCode != 0) {
                String errmsg = jsonObject.getString("errmsg");
                log.error("selectGroupMsgTask error msgId={} errCode={} errmsg={}", msgId, errCode, errmsg);
                return resList;
            }
            JSONArray taskList = jsonObject.getJSONArray("task_list");
            for (int i = 0; i < taskList.size(); i++) {
                JSONObject task = taskList.getJSONObject(i);
                String userId = task.getString("userid");
                resList.add(userId);
            }
            String nextCursor = jsonObject.getString("next_cursor");
            if (cursor.equals(nextCursor)) {
                break;
            }
            cursor = nextCursor;
        } while (StringUtils.isNotEmpty(cursor));
        return resList;
    }

    private String getGroupMsgTask(String msgId, String cursor, Long corpId) throws IOException {
        String accessToken = accessTokenService.getWeComAccessToken(corpId);
        String url = POST_WE_COM_GROUP_MSG_TASK
                + "?access_token=" + accessToken;
        JSONObject param = new JSONObject();
        param.put("msgid", msgId);
        param.put("limit", 1000);
        if (StringUtils.isNotEmpty(cursor)) {
            param.put("cursor", cursor);
        }
        return httpPoolClient.post(url, param.toJSONString());
    }

    private void delHistoryMessageList(Long startTime, Long endTime, Long corpId, Long staffId) {
        HistoryMessage historyMessage = new HistoryMessage();
        historyMessage.setIsDelete(1);
        HistoryMessageExample example = new HistoryMessageExample();
        HistoryMessageExample.Criteria criteria =
                example.createCriteria().andSendTimeBetween(new Date(startTime * 1000), new Date(endTime * 1000))
                        .andCorpIdEqualTo(corpId);
        if (staffId != null) {
            criteria.andStaffIdEqualTo(staffId);
        }
        historyMessageMapper.updateByExampleSelective(historyMessage, example);

    }

    private void insertHistoryMessageList(Staff staff, List<ExternalUser> externalUsers,
                                          List<MiniprogramRecord> miniprogramRecordList,
                                          Long createTime, Long corpId, Long startTime, Long endTime) {
        if (CollectionUtils.isEmpty(externalUsers) || CollectionUtils.isEmpty(miniprogramRecordList)) {
            return;
        }
        MessageAttachmentExample example = new MessageAttachmentExample();
        example.createCriteria().andStaffIdEqualTo(staff.getId())
                .andSendTimeBetween(new Date(startTime * 1000), new Date(endTime * 1000));
        List<MessageAttachment> messageAttachments = messageAttachmentMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(messageAttachments)) {
            LarkRobotUtil.sendMessage("获取保底set失败");
            return;
        }
        Set<Long> guaranteedSet = messageAttachments.stream()
                .map(MessageAttachment::getMiniprogramVideoId).collect(Collectors.toSet());
        Long sendTime = externalUsers.stream()
                .map(ExternalUser::getSendTime).filter(Objects::nonNull).findFirst().orElse(null);
        List<HistoryMessage> historyMessageList = new ArrayList<>();
        for (ExternalUser externalUser : externalUsers) {
            Long userId = userMapper.selectIdByExternalUserId(externalUser.getExternalUserId());
            if (userId == null) {
                continue;
            }
            for (MiniprogramRecord miniprogramRecord : miniprogramRecordList) {
                Integer status = externalUser.getStatus();
                HistoryMessage historyMessage = new HistoryMessage();
                if (externalUser.getSendTime() != null) {
                    historyMessage.setSendTime(new Date(externalUser.getSendTime()));
                } else {
                    if (sendTime != null) {
                        historyMessage.setSendTime(new Date(sendTime));
                    } else if (createTime != null) {
                        historyMessage.setSendTime(new Date(createTime));
                    }
                }
                Long videoId = miniprogramRecord.getVideoId();
                historyMessage.setAttachmentIdx(miniprogramRecord.getAttachmentIdx());
                historyMessage.setVideoId(videoId);
                historyMessage.setUserId(userId);
                historyMessage.setStaffId(staff.getId());
                historyMessage.setStatus(status);
                historyMessage.setCorpId(corpId);
                if (guaranteedSet.contains(videoId)) {
                    historyMessage.setSource(MANUAL.getName());
                } else {
                    historyMessage.setSource(HISTORICAL_TOP.getName());
                }
                historyMessageList.add(historyMessage);
                if (status == 2) {
                    userService.delStaffWithUser(userId, staff.getId(), sendTime);
                }
            }
        }
        historyMessageService.batchInsertHistoryMessage(historyMessageList);
    }

    private String getGroupMsgList(Long startTime, Long endTime, String cursor, Long corpId) throws IOException {
        String accessToken = accessTokenService.getWeComAccessToken(corpId);
        String url = POST_WE_COM_GROUP_MSG_LIST_URL
                + "?access_token=" + accessToken;
        JSONObject param = new JSONObject();
        param.put("chat_type", "single");
        param.put("start_time", startTime);
        param.put("end_time", endTime);
        param.put("limit", 100);
        if (StringUtils.isNotEmpty(cursor)) {
            param.put("cursor", cursor);
        }
        return httpPoolClient.post(url, param.toJSONString());
    }

    @XxlJob("insetHistoryMessageSourceJob")
    public ReturnT<String> insetHistoryMessageSource(String param) {
        if (StringUtils.isEmpty(param)) {
            return ReturnT.FAIL;
        }
        JSONArray jsonArray = JSONArray.parseArray(param);
        for (int i = 0; i < jsonArray.size(); i++) {
            JSONObject jsonObject = jsonArray.getJSONObject(i);
            Long startTime = jsonObject.getLong("startTime");
            Long endTime = jsonObject.getLong("endTime");

            JSONArray videos = jsonObject.getJSONArray("videos");
            List<Long> videoList = videos.stream().map(String::valueOf).map(Long::parseLong).collect(Collectors.toList());
            HistoryMessageExample example = new HistoryMessageExample();
            example.createCriteria().andSendTimeBetween(new Date(startTime), new Date(endTime))
                    .andVideoIdIn(videoList);
            HistoryMessage historyMessage = new HistoryMessage();
            historyMessage.setSource(MANUAL.getName());
            historyMessageMapper.updateByExampleSelective(historyMessage, example);

            HistoryMessageExample example1 = new HistoryMessageExample();
            example1.createCriteria().andSendTimeBetween(new Date(startTime), new Date(endTime))
                    .andVideoIdNotIn(videoList);
            HistoryMessage historyMessage1 = new HistoryMessage();
            historyMessage1.setSource(HISTORICAL_TOP.getName());
            historyMessageMapper.updateByExampleSelective(historyMessage1, example1);
        }
        return ReturnT.SUCCESS;
    }
}




